#!/usr/bin/perl -w

# Moblin Application Checker
# Results Page Module (tests_results.pl)
#
# Copyright (C) 2007-2009 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# version 2 as published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;
use Fcntl ':flock';
use Fcntl ':seek';

autoflush_on();

#-----------------------------------------------------------------------

if ($_POST{"remove"}) {
	if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
		flock(FILE, LOCK_EX);
		my @list;
		my %decoded = ();
		foreach (keys %_POST) {
			my $key = $_;
			$key =~ s/\./%/g;
			$decoded{unescape($key)} = $_POST{$_};
		}
		while (<FILE>) {
			s/[\r\n]//g;
			next if ($_ eq "");
			my $path = $CONFIG{"RESULTS_DIR"}."/$_";
			if (-d $path) {
				if (defined($decoded{$_})) {
					system('rm -rf '.escape_path($path));
				}
				else {
					push @list, $_;
				}
			}
		}
		seek(FILE, 0, SEEK_SET);
		truncate(FILE, 0);
		print FILE join("\n", @list)."\n";
		flock(FILE, LOCK_UN);
		close(FILE);
	}
	else {
		print STDERR "Cannot open HISTORY file!\n";
	}
	print 'HTTP/1.0 302 Moved' . CRLF;
	print 'Location: tests_results.pl' . CRLF . CRLF;
	exit;
}

sub http_header {
	my ($add) = @_;
	
	my $res = "";
	
	$res .= 'HTTP/1.0 200 OK' . CRLF;
	$res .= 'Content-type: text/html' . CRLF;
	if ( $add ) {
		$add =~ s/[\r\n]+/${\CRLF}/g;
		$res .= $add;
	}
	$res .= CRLF; # Empty line finishes the header
	
	return $res;
}

my $popup_html_header = <<HEREDOC;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
<head>
	<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
	<title>Details</title>
    
	<style type="text/css">
		body {
			font: small sans-serif;
		}
		a {
			text-decoration: none;
		}
		a:hover {
			text-decoration: underline;
		}
		a[href ^="http://"] {
			background: url(css/external.png) center right no-repeat;
			target-name: new; target-new: window; target-position: above;
			padding-right: 13px;
		}
		div.box {
			border: 1px solid #aaa;
			background-color: #f9f9f9;
			padding: 5pt;
			padding-left: 15pt;
		}
		table {
			border: 1px #aaaaaa solid;
			border-collapse: collapse;
			background: #f9f9f9;
		}
		table th, table td {
			border: 1px #aaaaaa solid;
			padding: 0.2em 0.4em;
		}
		table th {
			background: #f2f2f2;
		}
	</style>
</head>
<body>
HEREDOC

my $popup_html_footer = '</body></html>';

# Reads information about the tested application
sub read_results_info($) {
	my ($res_dir) = @_;

	my $app_name = "";
	if (open(FILE, "$res_dir/INFO")) {
		while (<FILE>) {
			if (m/Application: (.*)$/) {
				$app_name = $1;
				last;
			}
		}
		close(FILE);
	}
	my $res_dir_esc = $res_dir;
	$res_dir_esc =~ s/([ ()\'])/\\$1/g;
	my $host_platform = `cat $res_dir_esc/test_logs/host_platform`;
	my $moblin_version = `cat $res_dir_esc/test_logs/moblin_version`;
	$host_platform =~ s/[\r\n]//g;
	$moblin_version =~ s/[\r\n]//g;

	return ($app_name, $host_platform, $moblin_version);
}

sub tab_link ($) {
	my ($tab) = @_;
	
	return '?details='.$_GET{'details'}.'&tab='.$tab;
}

sub href ($) {
	my ($link) = @_;
	
	$link =~ s/\+/%2B/g;
	$link =~ s/ /%20/g;
	
	return $link;
}

sub html ($) {
	my ($txt) = @_;
	
	$txt =~ s/&/&amp;/g;
	$txt =~ s/</&lt;/g;
	$txt =~ s/>/&gt;/g;
	$txt =~ s/\"/&quot;/g;
	
	return $txt;
}

my @res_tab_ids = ('cert');
my %res_tab_names = (
	'cert'   => 'Moblin Certification',
);

# Customizable tab bar
sub html_results_tabs {
	my ($selected) = @_;

	my $nl = "\n";
	my $res = "";

	# Table start
	$res .= '<table cellpadding="0" cellspacing="0" border="0" width="100%"'
		.' style="white-space: nowrap; font-weight: bold; text-align: center; font-size: 120%; margin-bottom: 5pt;">'.$nl;
	$res .= '<tr>'.$nl;

	# Left spacer
	$res .= '<td style="width: 10px; border-bottom: solid 1px #166a98; height: 2em;">';
	$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
	$res .= '</td>'.$nl;

	my $tabs_num = scalar(@res_tab_ids);

	foreach my $id (@res_tab_ids) {
		my $name = $res_tab_names{$id};

		# Tab
		if ( $id eq $selected ) {
			$res .= '<td class="res-tab-selected">';
		} else {
			$res .= '<td class="res-tab-highlight">';
			$res .= '<a class="res-tab-link" href="'.html(href(tab_link($id))).'">';
		}
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '<img src="images/res-'.$id.'.png" width="16" height="16" alt="" />&nbsp;';
		$name =~ s/\s/&nbsp;/g;
		$res .= $name;
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '</a>' if $id ne $selected;
		$res .= '</td>';

		# Spacer
		$res .= '<td style="width: 10px; border-bottom: solid 1px #166a98;">';
		$res .= '<img src="images/environment-spacer.gif" width="10" height="1" alt="" />';
		$res .= '</td>'.$nl;
	}

	$res .= '<td style="border-bottom: solid 1px #166a98; width: 100%;">';
	$res .= '<img src="images/environment-spacer.gif" width="1" height="1" alt="" />';
	$res .= '</td>';

	$res .= '</tr>';

	# Bottom line
	$res .= '<tr>';
	$res .= '<td colspan="'.(2 * $tabs_num+2).'" style="background-color: #a0d5f1; height: 10px;">';
	$res .= '<img src="images/environment-spacer.gif" width="1" height="1" alt="" />';
	$res .= '</td>';
	$res .= '</tr>';

	$res .= '</table>'.$nl;

	return $res;
}

sub html_title {
	my ($app_name, $architecture, $res_dir) = @_;

	my $res = '';
	if (! -f "$res_dir/test_logs/ftp_upload") {
		my $href_name = $_GET{'details'};
		$href_name =~ s/([^0-9a-zA-Z_-])/sprintf('%%%02X', ord($1))/eg;
		$res .= <<DATA;
<table border="0" cellpadding="0" cellspacing="0" class="upload-suggest" onclick="javascript:document.location='upload_data.pl?details=$href_name';">
  <tr>
    <td valign="middle"><img src="images/upload_data.png" width="16" height="16" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="3" height="1" alt="" /></td>
    <td>Please <a href="upload_data.pl?details=$href_name" onclick="javascript:event.cancelBubble=true;"><b>upload</b></a> info about your application to the <a href="http://www.linuxfoundation.org/navigator/browse/app.php" target="_blank" onclick="javascript:alert('Moblin Navigator is not yet available.'); event.cancelBubble=true; return false;">Moblin Navigator</a>.</td>
  </tr>
</table>
<script type="text/javascript" language="javascript">
// <![CDATA[
preload_images(new Array('ajax_progress.gif'));
// ]]>
</script>
DATA
	$res .= "<h1>Analysis Results for <font color=\"blue\">$app_name</font> on <font color=\"blue\">$architecture</font></h1>\n\n";

	}
	return $res;
}

if ($_GET{"details"}) {
	my $default_open = !$_GET{'tab'};
	$_GET{'tab'} = 'cert' if (!$_GET{'tab'});
	my $res_dir = $CONFIG{"RESULTS_DIR"}."/".$_GET{"details"};
	my ($app_name, $host_platform, $moblin_version) = read_results_info($res_dir);

	{
		if ($err) {
			print http_header();
			print_header("App Analysis", "res-error");
		
			print '<p style="color:red">';
			print $err;
			print '</p>';
			
			# 'View test log' link:
			my $link = href('/get.pl'.$SERVER_PARAM{'APP_DATA'}.'/results/'.$_GET{'details'}.'/test_logs/tests_output');
			print '<p><a href="'.html($link).'" target="_blank">View the test log</a></p>';
			
			print '<form name="del_form" action="tests_results.pl" method="post">';
			print '<input type="hidden" name="remove" value="1"/>';
			print '<input type="hidden" name="'.html($_GET{'details'}).'" value="1"/>';
			print '</form>';
			print '<p>[<a href="" onclick="javascript:document.del_form.submit(); return false;">Delete the result</a>]</p>';
			
			print_footer();
			exit 0;
		}
	}
	
	local *header_info_tabs = sub { # Render the header, info block and the tabs ruller.
		return html_title( $app_name, $host_platform, $res_dir )
				.html_results_tabs( $_GET{'tab'} )
			;
	};

	if ($_GET{"tab"} eq 'cert') {					# Certification Tab
		my $name = "test_log.html";
		print http_header();
		print_header("App Analysis", "res-cert");
		if (open(FILE, $CONFIG{"RESULTS_DIR"}."/".$_GET{"details"}."/$name")) {
			my $transfer = 0;
			while (<FILE>) {
				if (m/<body>/) {
					s/^.*<body>\n?//i;
					$transfer = 1;
				}
				if ($transfer) {
					s/(<\/body>|<\/html>)\n?//i;
					if (m/<!-- Test Report Header -->/) {
						print header_info_tabs();
						print <<DATA;
<h1>Moblin $moblin_version Test Results</h1>

<form action="cert_integration.pl?start" method="post" name="integr_data" style="display:none">
<input type="hidden" name="file" value="$CONFIG{'RESULTS_DIR'}/$_GET{'details'}/$_GET{'details'}.tgz" />
</form>
<p style="margin-top: 1.5em;"><a href="cert_integration.pl?start" onclick="javascript:document.integr_data.submit(); return false"><img src="images/medal_small.jpg" width="28" height="28" alt="" /></a> If there are no errors, you can easily <a href="cert_integration.pl?start" onclick="document.integr_data.submit(); return false"><b>apply for certification</b></a> by uploading the results to the <a href="http://$cert_sys_host$cert_sys_base" target="_blank">Moblin Certification System</a>.</p>

DATA
					}
					else {
						print $_;
					}
				}
			}
			close(FILE);
		}
		else {
			print "<b><font color=\"red\">Sorry, the report '".$_GET{"details"}."' cannot be opened!</font><br />\nReason: $!</b>\n";
		}
		print_footer();
	} # end of the 'cert' tab
	else {
		print_header("App Analysis", "res-distrs");
		print "<b><font color=\"red\">Invalid tab selected!</font></b>\n";
		print_footer();
	}
} # end of  if $_GET{'details'}
else {
	print http_header();
	print_header("Test Results", "results");

	print "<h1>Test Results</h1>\n";

	print "<form method=\"post\" action=\"tests_results.pl\" name=\"results_list\">\n<table class=\"main\">\n";
	print "  <tr>\n    <td><input type=\"checkbox\" name=\"check_all\" id=\"check_all\" onclick=\"javascript:check_unckeck_all();\" /></td>\n";
	print "    <th style=\"width: 14em;\">Date/Time</th>\n";
	print "    <th style=\"width: 30em;\">Comment</th>\n  </tr>\n";
	my @list = ();
	if (open(FILE, "+<".$CONFIG{"RESULTS_DIR"}."/HISTORY")) {
		flock(FILE, LOCK_EX);
		while(<FILE>) {
			s/[\r\n]//g;
			next if ($_ eq "");
			push @list, $_ if (-d $CONFIG{"RESULTS_DIR"}."/$_");
		}
		seek(FILE, 0, SEEK_SET);
		truncate(FILE, 0);
		print FILE join("\n", @list)."\n";
		flock(FILE, LOCK_UN);
		close(FILE);
		@list = reverse(@list);
		foreach my $name (@list) {
			my $full_name = $CONFIG{"RESULTS_DIR"}."/$name";
			my $href_name = $name;
			$href_name =~ s/([^0-9a-zA-Z_-])/sprintf('%%%02X', ord($1))/eg;
			my $id_name = $href_name;
			$id_name =~ s/%/./g;
			my $result = 'unknown';
			my $app_name = "";
			my @comment = ();
			my $reading_comment = 0;
			if (open(FILE, "$full_name/INFO")) {
				while (<FILE>) {
					s/[\r\n]//g;
					if ($reading_comment) {
						push @comment, $_;
					}
					else {
						if ($_ eq '') {
							$reading_comment = 1;
						}
						elsif (m/^Verdict: (\S+)/i) {
							if ($1 eq "Success") {
								$result = 'pass';
							}
							elsif ($1 eq "Warnings") {
								$result = 'warn';
							}
							elsif ($1 eq "Failed") {
								$result = 'fail';
							}
						}
						elsif (m/^Application: (.*)$/i) {
							$app_name = $1;
						}
					}
				}
				close(FILE);
			}
#			if (open(FILE, "$full_name/INFO_MOBLIN")) {
#				while (<FILE>) {
#					s/[\r\n]//g;
#					if (m/^Verdict: (Success|Warnings)/i) {
#						$result .= '-moblin';
#					}
#				}
#				close(FILE);
#			}
			my @extraction = ();
			my $jcomment = join("<br />", @comment);
			if ($jcomment eq "") {
				$jcomment = "&nbsp;";
			}
			my $show_name = $name;
			$show_name =~ s/^.*-([^\-]*)\-([^\-]*)\-([^\-]*)\-([^\-]*)h\-([^\-]*)m\-([^\-]*)s/$3.$2.$1&nbsp;$4:$5:$6/;
			my $upload_icon = "";
			if (! -f $CONFIG{"RESULTS_DIR"}."/$name/test_logs/ftp_upload") {
				$upload_icon = "&nbsp;<a href=\"upload_data.pl?details=$href_name\" title=\"Upload Data to the Linux Foundation\"><img src=\"images/upload_data.png\" alt=\"\" width=\"16\" height=\"16\" /></a>";
			}
			print <<DATA;
  <tr>
    <td rowspan="3"><input type='checkbox' name='$id_name' onclick='javascript:update_state();' /></td>
    <td rowspan="1" style="border-bottom: none;"><img src="images/environment-spacer.gif" width="1" height="16" alt="" /></td>
    <td rowspan="3" valign="top">$jcomment</td>
  </tr>
  <tr>
    <td valign="middle" style="border: none;">
      <table cellpadding="0" cellspacing="0" border="0"><tr>
        <td style="border: none; padding: 0px;"><img src="images/result-$result.png" alt="$result" width="25" height="41" /></td>
        <td style="border: none; padding: 0px;">&nbsp;<a href="tests_results.pl?details=$href_name">$show_name</a><br />&nbsp;Application: $app_name</td>
      </tr></table>
    </td>
  </tr>
  <tr>
    <td style="border: none; text-align: right; vertical-align: bottom;" width="5" height="5">
      <!--a href="tests_results.pl?details=$href_name&amp;tab=distrs" title="Show distribution compatibility report"><img src="images/res-distrs.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=com" title="Show application components"><img src="images/res-com.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=libs" title="Show required external libraries"><img src="images/res-libs.png" alt="" width="16" height="16" /></a>&nbsp;<a href="tests_results.pl?details=$href_name&amp;tab=ints" title="Show required external interfaces"><img src="images/res-ints.png" alt="" width="16" height="16" /></a>&nbsp;--><a href="tests_results.pl?details=$href_name&amp;tab=cert" title="Show Moblin compliance report"><img src="images/res-cert.png" alt="" width="16" height="16" /></a>$upload_icon
    </td>
  </tr>
DATA
		}
	}
	print <<DATA;
</table>
<br /><input type="submit" name="remove" id="remove_button" value="Remove Selected Entries" disabled="disabled" onclick='javascript:return confirm_remove();' />
</form>

<script language="javascript" type="text/javascript">
// <![CDATA[
function count_checked() {
	var num = 0;
	var form = document.results_list;
	for (var i=0; i<form.length; ++i) {
		if ((form[i].type.toLowerCase() == "checkbox") && (form[i].name != "check_all") && form[i].checked) {
			++num;
		}
	}
	return num;
}

function update_state() {
	var elem = document.getElementById("remove_button");
	if (elem) {
		var num = count_checked();
		elem.disabled = (num == 0);
	}
}

function check_unckeck(box, check) {
	temp = box.onchange;
	box.onchange = null;
	box.checked = check;
	box.onchange = temp;
}

function check_unckeck_all() {
	var elem = document.getElementById("check_all");
	if (elem) {
		var checked = elem.checked;
		var form = document.results_list;
		for (var i=0; i<form.length; ++i) {
			if ((form[i].type.toLowerCase() == "checkbox") && (form[i].name != "check_all"))
				check_unckeck(form[i], checked);
		}
		update_state();
	}
}

function confirm_remove() {
	var num = count_checked();
	if (num == 0) {
		alert("Nothing selected!");
		return false
	}
	else
		return confirm("Do you wish to delete " + num + " selected report(s)?");
}
// ]]>
</script>
DATA
	print_footer();
} # end of the result list (result history)

exit 0;

__END__
